@@ -12,13 +12,13 @@ import sys
 import os.path as osp
 
 
-__version__ = '2.1.8'
+__version__ = '3.0.7'
 
 
 #{ Initialization
 def _init_externals():
     """Initialize external projects by putting them into the path"""
-    if __version__ == '2.1.8':
+    if __version__ == '3.0.7':
         sys.path.insert(0, osp.join(osp.dirname(__file__), 'ext', 'gitdb'))
 
     try:
@@ -79,5 +79,8 @@ def refresh(path=None):
 #} END initialize git executable path
 
 #################
-refresh()
+try:
+    refresh()
+except Exception as exc:
+    raise ImportError('Failed to initialize: {0}'.format(exc))
 #################
