@@ -3,7 +3,6 @@ import os
 import stat
 from string import digits
 
-from git.compat import xrange
 from git.exc import WorkTreeRepositoryUnsupported
 from git.objects import Object
 from git.refs import SymbolicReference
@@ -86,7 +85,7 @@ def find_submodule_git_dir(d):
                 ## Cygwin creates submodules prefixed with `/cygdrive/...` suffixes.
                 path = decygpath(path)
             if not osp.isabs(path):
-                path = osp.join(osp.dirname(d), path)
+                path = osp.normpath(osp.join(osp.dirname(d), path))
             return find_submodule_git_dir(path)
     # end handle exception
     return None
@@ -307,7 +306,7 @@ def rev_parse(repo, rev):
         try:
             if token == "~":
                 obj = to_commit(obj)
-                for _ in xrange(num):
+                for _ in range(num):
                     obj = obj.parents[0]
                 # END for each history item to walk
             elif token == "^":
