@@ -11,23 +11,18 @@ import gc
 import io
 import logging
 import os
-import sys
 import tempfile
 import textwrap
 import time
+import unittest
 
-from git.compat import string_types, is_win
+from git.compat import is_win
 from git.util import rmtree, cwd
 import gitdb
 
 import os.path as osp
 
 
-if sys.version_info[0:2] == (2, 6):
-    import unittest2 as unittest
-else:
-    import unittest
-
 TestCase = unittest.TestCase
 SkipTest = unittest.SkipTest
 skipIf = unittest.skipIf
@@ -121,7 +116,7 @@ def with_rw_repo(working_tree_ref, bare=False):
     To make working with relative paths easier, the cwd will be set to the working
     dir of the repository.
     """
-    assert isinstance(working_tree_ref, string_types), "Decorator requires ref name for working tree checkout"
+    assert isinstance(working_tree_ref, str), "Decorator requires ref name for working tree checkout"
 
     def argument_passer(func):
         @wraps(func)
@@ -252,7 +247,7 @@ def with_rw_and_rw_remote_repo(working_tree_ref):
     """
     from git import Git, Remote  # To avoid circular deps.
 
-    assert isinstance(working_tree_ref, string_types), "Decorator requires ref name for working tree checkout"
+    assert isinstance(working_tree_ref, str), "Decorator requires ref name for working tree checkout"
 
     def argument_passer(func):
 
@@ -348,12 +343,6 @@ class TestBase(TestCase):
       of the project history ( to assure tests don't fail for others ).
     """
 
-    # On py26, unittest2 has assertRaisesRegex
-    # On py3, unittest has assertRaisesRegex
-    # On py27, we use unittest, which names it differently:
-    if sys.version_info[0:2] == (2, 7):
-        assertRaisesRegex = TestCase.assertRaisesRegexp
-
     def _small_repo_url(self):
         """:return" a path to a small, clonable repository"""
         from git.cmd import Git
