@@ -2,14 +2,9 @@ from io import BytesIO
 from stat import S_IFDIR, S_IFREG, S_IFLNK
 from os import stat
 import os.path as osp
-
-try:
-    from unittest import skipIf, SkipTest
-except ImportError:
-    from unittest2 import skipIf, SkipTest
+from unittest import SkipTest
 
 from git import Git
-from git.compat import PY3
 from git.index import IndexFile
 from git.index.fun import (
     aggressive_tree_merge
@@ -215,7 +210,7 @@ class TestFun(TestBase):
     def _assert_tree_entries(self, entries, num_trees):
         for entry in entries:
             assert len(entry) == num_trees
-            paths = set(e[2] for e in entry if e)
+            paths = {e[2] for e in entry if e}
 
             # only one path per set of entries
             assert len(paths) == 1
@@ -286,12 +281,6 @@ class TestFun(TestBase):
         statbuf = stat(gitdir)
         assert_true(statbuf.st_mode & S_IFDIR)
 
-    @skipIf(PY3, 'odd types returned ... maybe figure it out one day')
-    def test_tree_entries_from_data_with_failing_name_decode_py2(self):
-        r = tree_entries_from_data(b'100644 \x9f\0aaa')
-        assert r == [('aaa', 33188, u'\udc9f')], r
-
-    @skipIf(not PY3, 'odd types returned ... maybe figure it out one day')
     def test_tree_entries_from_data_with_failing_name_decode_py3(self):
         r = tree_entries_from_data(b'100644 \x9f\0aaa')
         assert r == [(b'aaa', 33188, '\udc9f')], r
