@@ -7,6 +7,7 @@
 import os
 import subprocess
 import sys
+from tempfile import TemporaryFile
 
 from git import (
     Git,
@@ -16,7 +17,7 @@ from git import (
     Repo,
     cmd
 )
-from git.compat import PY3, is_darwin
+from git.compat import is_darwin
 from git.test.lib import (
     TestBase,
     patch,
@@ -60,18 +61,12 @@ class TestGit(TestBase):
 
     def test_call_unpack_args_unicode(self):
         args = Git._Git__unpack_args(u'Unicode€™')
-        if PY3:
-            mangled_value = 'Unicode\u20ac\u2122'
-        else:
-            mangled_value = 'Unicode\xe2\x82\xac\xe2\x84\xa2'
+        mangled_value = 'Unicode\u20ac\u2122'
         assert_equal(args, [mangled_value])
 
     def test_call_unpack_args(self):
         args = Git._Git__unpack_args(['git', 'log', '--', u'Unicode€™'])
-        if PY3:
-            mangled_value = 'Unicode\u20ac\u2122'
-        else:
-            mangled_value = 'Unicode\xe2\x82\xac\xe2\x84\xa2'
+        mangled_value = 'Unicode\u20ac\u2122'
         assert_equal(args, ['git', 'log', '--', mangled_value])
 
     @raises(GitCommandError)
@@ -81,17 +76,20 @@ class TestGit(TestBase):
     def test_it_transforms_kwargs_into_git_command_arguments(self):
         assert_equal(["-s"], self.git.transform_kwargs(**{'s': True}))
         assert_equal(["-s", "5"], self.git.transform_kwargs(**{'s': 5}))
+        assert_equal([], self.git.transform_kwargs(**{'s': None}))
 
         assert_equal(["--max-count"], self.git.transform_kwargs(**{'max_count': True}))
         assert_equal(["--max-count=5"], self.git.transform_kwargs(**{'max_count': 5}))
+        assert_equal(["--max-count=0"], self.git.transform_kwargs(**{'max_count': 0}))
+        assert_equal([], self.git.transform_kwargs(**{'max_count': None}))
 
         # Multiple args are supported by using lists/tuples
         assert_equal(["-L", "1-3", "-L", "12-18"], self.git.transform_kwargs(**{'L': ('1-3', '12-18')}))
-        assert_equal(["-C", "-C"], self.git.transform_kwargs(**{'C': [True, True]}))
+        assert_equal(["-C", "-C"], self.git.transform_kwargs(**{'C': [True, True, None, False]}))
 
         # order is undefined
         res = self.git.transform_kwargs(**{'s': True, 't': True})
-        self.assertEqual(set(['-s', '-t']), set(res))
+        self.assertEqual({'-s', '-t'}, set(res))
 
     def test_it_executes_git_to_shell_and_returns_result(self):
         assert_match(r'^git version [\d\.]{2}.*$', self.git.execute(["git", "version"]))
@@ -108,6 +106,11 @@ class TestGit(TestBase):
         self.git.version(pass_this_kwarg=False)
         assert_true("pass_this_kwarg" not in git.call_args[1])
 
+    @raises(GitCommandError)
+    def test_it_raises_proper_exception_with_output_stream(self):
+        tmp_file = TemporaryFile()
+        self.git.checkout('non-existent-branch', output_stream=tmp_file)
+
     def test_it_accepts_environment_variables(self):
         filename = fixture_path("ls_tree_empty")
         with open(filename, 'r') as fh:
@@ -125,15 +128,18 @@ class TestGit(TestBase):
 
     def test_persistent_cat_file_command(self):
         # read header only
-        import subprocess as sp
         hexsha = "b2339455342180c7cc1e9bba3e9f181f7baa5167"
-        g = self.git.cat_file(batch_check=True, istream=sp.PIPE, as_process=True)
+        g = self.git.cat_file(
+            batch_check=True, istream=subprocess.PIPE, as_process=True
+        )
         g.stdin.write(b"b2339455342180c7cc1e9bba3e9f181f7baa5167\n")
         g.stdin.flush()
         obj_info = g.stdout.readline()
 
         # read header + data
-        g = self.git.cat_file(batch=True, istream=sp.PIPE, as_process=True)
+        g = self.git.cat_file(
+            batch=True, istream=subprocess.PIPE, as_process=True
+        )
         g.stdin.write(b"b2339455342180c7cc1e9bba3e9f181f7baa5167\n")
         g.stdin.flush()
         obj_info_two = g.stdout.readline()
@@ -151,7 +157,7 @@ class TestGit(TestBase):
 
         # same can be achieved using the respective command functions
         hexsha, typename, size = self.git.get_object_header(hexsha)
-        hexsha, typename_two, size_two, data = self.git.get_object_data(hexsha)  # @UnusedVariable
+        hexsha, typename_two, size_two, _ = self.git.get_object_data(hexsha)
         self.assertEqual(typename, typename_two)
         self.assertEqual(size, size_two)
 
@@ -255,7 +261,7 @@ class TestGit(TestBase):
                 remote.fetch()
             except GitCommandError as err:
                 if sys.version_info[0] < 3 and is_darwin:
-                    self.assertIn('ssh-orig, ' in str(err))
+                    self.assertIn('ssh-orig', str(err))
                     self.assertEqual(err.status, 128)
                 else:
                     self.assertIn('FOO', str(err))
