@@ -11,12 +11,8 @@ from stat import (
     S_ISLNK,
     ST_MODE
 )
-import sys
 import tempfile
-try:
-    from unittest import skipIf
-except ImportError:
-    from unittest2 import skipIf
+from unittest import skipIf
 
 from git import (
     IndexFile,
@@ -29,7 +25,7 @@ from git import (
     GitCommandError,
     CheckoutError,
 )
-from git.compat import string_types, is_win, PY3
+from git.compat import is_win
 from git.exc import (
     HookExecutionError,
     InvalidGitRepositoryError
@@ -79,7 +75,7 @@ class TestIndex(TestBase):
 
     def _assert_fprogress(self, entries):
         self.assertEqual(len(entries), len(self._fprogress_map))
-        for path, call_count in self._fprogress_map.items():  # @UnusedVariable
+        for _path, call_count in self._fprogress_map.items():
             self.assertEqual(call_count, 2)
         # END for each item in progress map
         self._reset_progress()
@@ -101,7 +97,7 @@ class TestIndex(TestBase):
 
     def _reset_progress(self):
         # maps paths to the count of calls
-        self._fprogress_map = dict()
+        self._fprogress_map = {}
 
     def _assert_entries(self, entries):
         for entry in entries:
@@ -131,7 +127,7 @@ class TestIndex(TestBase):
         # test stage
         index_merge = IndexFile(self.rorepo, fixture_path("index_merge"))
         self.assertEqual(len(index_merge.entries), 106)
-        assert len(list(e for e in index_merge.entries.values() if e.stage != 0))
+        assert len([e for e in index_merge.entries.values() if e.stage != 0])
 
         # write the data - it must match the original
         tmpfile = tempfile.mktemp()
@@ -145,14 +141,14 @@ class TestIndex(TestBase):
         if isinstance(tree, str):
             tree = self.rorepo.commit(tree).tree
 
-        blist = list()
+        blist = []
         for blob in tree.traverse(predicate=lambda e, d: e.type == "blob", branch_first=False):
             assert (blob.path, 0) in index.entries
             blist.append(blob)
         # END for each blob in tree
         if len(blist) != len(index.entries):
-            iset = set(k[0] for k in index.entries.keys())
-            bset = set(b.path for b in blist)
+            iset = {k[0] for k in index.entries.keys()}
+            bset = {b.path for b in blist}
             raise AssertionError("CMP Failed: Missing entries in index: %s, missing in tree: %s" %
                                  (bset - iset, iset - bset))
         # END assertion message
@@ -168,9 +164,7 @@ class TestIndex(TestBase):
         except Exception as ex:
             msg_py3 = "required argument is not an integer"
             msg_py2 = "cannot convert argument to integer"
-            msg_py26 = "unsupported operand type(s) for &: 'str' and 'long'"
-            assert msg_py2 in str(ex) or msg_py3 in str(ex) or \
-                msg_py26 in str(ex), str(ex)
+            assert msg_py2 in str(ex) or msg_py3 in str(ex)
 
         ## 2nd time should not fail due to stray lock file
         try:
@@ -180,9 +174,6 @@ class TestIndex(TestBase):
 
     @with_rw_repo('0.1.6')
     def test_index_file_from_tree(self, rw_repo):
-        if sys.version_info < (2, 7):
-            ## Skipped, not `assertRaisesRegexp` in py2.6
-            return
         common_ancestor_sha = "5117c9c8a4d3af19a9958677e45cda9269de1541"
         cur_sha = "4b43ca7ff72d5f535134241e7c797ddc9c7a3573"
         other_sha = "39f85c4358b7346fee22169da9cad93901ea9eb9"
@@ -199,7 +190,7 @@ class TestIndex(TestBase):
 
         # merge three trees - here we have a merge conflict
         three_way_index = IndexFile.from_tree(rw_repo, common_ancestor_sha, cur_sha, other_sha)
-        assert len(list(e for e in three_way_index.entries.values() if e.stage != 0))
+        assert len([e for e in three_way_index.entries.values() if e.stage != 0])
 
         # ITERATE BLOBS
         merge_required = lambda t: t[0] != 0
@@ -210,7 +201,7 @@ class TestIndex(TestBase):
 
         # test BlobFilter
         prefix = 'lib/git'
-        for stage, blob in base_index.iter_blobs(BlobFilter([prefix])):  # @UnusedVariable
+        for _stage, blob in base_index.iter_blobs(BlobFilter([prefix])):
             assert blob.path.startswith(prefix)
 
         # writing a tree should fail with an unmerged index
@@ -235,7 +226,7 @@ class TestIndex(TestBase):
     def test_index_merge_tree(self, rw_repo):
         # A bit out of place, but we need a different repo for this:
         self.assertNotEqual(self.rorepo, rw_repo)
-        self.assertEqual(len(set((self.rorepo, self.rorepo, rw_repo, rw_repo))), 2)
+        self.assertEqual(len({self.rorepo, self.rorepo, rw_repo, rw_repo}), 2)
 
         # SINGLE TREE MERGE
         # current index is at the (virtual) cur_commit
@@ -397,7 +388,7 @@ class TestIndex(TestBase):
             self.assertEqual(len(e.failed_files), 1)
             self.assertEqual(e.failed_files[0], osp.basename(test_file))
             self.assertEqual(len(e.failed_files), len(e.failed_reasons))
-            self.assertIsInstance(e.failed_reasons[0], string_types)
+            self.assertIsInstance(e.failed_reasons[0], str)
             self.assertEqual(len(e.valid_files), 0)
             with open(test_file, 'rb') as fd:
                 s = fd.read()
@@ -536,7 +527,7 @@ class TestIndex(TestBase):
 
         # same index, no parents
         commit_message = "index without parents"
-        commit_no_parents = index.commit(commit_message, parent_commits=list(), head=True)
+        commit_no_parents = index.commit(commit_message, parent_commits=[], head=True)
         self.assertEqual(commit_no_parents.message, commit_message)
         self.assertEqual(len(commit_no_parents.parents), 0)
         self.assertEqual(cur_head.commit, commit_no_parents)
@@ -781,7 +772,16 @@ class TestIndex(TestBase):
             orig_tree = commit.tree
             self.assertEqual(index.write_tree(), orig_tree)
         # END for each commit
-
+    
+    @with_rw_repo('HEAD', bare=False)
+    def test_index_single_addremove(self, rw_repo):
+        fp = osp.join(rw_repo.working_dir, 'testfile.txt')
+        with open(fp, 'w') as fs:
+            fs.write(u'content of testfile')
+        self._assert_entries(rw_repo.index.add(fp))
+        deleted_files = rw_repo.index.remove(fp)
+        assert deleted_files
+        
     def test_index_new(self):
         B = self.rorepo.tree("6d9b1f4f9fa8c9f030e3207e7deacc5d5f8bba4e")
         H = self.rorepo.tree("25dca42bac17d511b7e2ebdd9d1d679e7626db5f")
@@ -821,10 +821,6 @@ class TestIndex(TestBase):
             asserted = True
         assert asserted, "Adding using a filename is not correctly asserted."
 
-    @skipIf(HIDE_WINDOWS_KNOWN_ERRORS and not PY3, r"""
-        FIXME:  File "C:\projects\gitpython\git\util.py", line 125, in to_native_path_linux
-        return path.replace('\\', '/')
-        UnicodeDecodeError: 'ascii' codec can't decode byte 0xc3 in position 0: ordinal not in range(128)""")
     @with_rw_directory
     def test_add_utf8P_path(self, rw_dir):
         # NOTE: fp is not a Unicode object in python 2 (which is the source of the problem)
@@ -847,6 +843,21 @@ class TestIndex(TestBase):
         r.index.add([fp])
         r.index.commit('Added [.exe')
 
+    def test__to_relative_path_at_root(self):
+        root = osp.abspath(os.sep)
+
+        class Mocked(object):
+            bare = False
+            git_dir = root
+            working_tree_dir = root
+
+        repo = Mocked()
+        path = os.path.join(root, 'file')
+        index = IndexFile(repo)
+
+        rel = index._to_relative_path(path)
+        self.assertEqual(rel, os.path.relpath(path, root))
+
     @with_rw_repo('HEAD', bare=True)
     def test_pre_commit_hook_success(self, rw_repo):
         index = rw_repo.index
@@ -891,10 +902,10 @@ class TestIndex(TestBase):
         _make_hook(
             index.repo.git_dir,
             'commit-msg',
-            'echo -n " {0}" >> "$1"'.format(from_hook_message)
+            'printf " {}" >> "$1"'.format(from_hook_message)
         )
         new_commit = index.commit(commit_message)
-        self.assertEqual(new_commit.message, u"{0} {1}".format(commit_message, from_hook_message))
+        self.assertEqual(new_commit.message, u"{} {}".format(commit_message, from_hook_message))
 
     @with_rw_repo('HEAD', bare=True)
     def test_commit_msg_hook_fail(self, rw_repo):
