@@ -6,10 +6,7 @@
 
 import random
 import tempfile
-try:
-    from unittest import skipIf
-except ImportError:
-    from unittest2 import skipIf
+from unittest import skipIf
 
 from git import (
     RemoteProgress,
@@ -25,7 +22,6 @@ from git import (
     GitCommandError
 )
 from git.cmd import Git
-from git.compat import string_types
 from git.test.lib import (
     TestBase,
     with_rw_repo,
@@ -34,7 +30,7 @@ from git.test.lib import (
     GIT_DAEMON_PORT,
     assert_raises
 )
-from git.util import IterableList, rmtree, HIDE_WINDOWS_FREEZE_ERRORS
+from git.util import rmtree, HIDE_WINDOWS_FREEZE_ERRORS
 import os.path as osp
 
 
@@ -47,8 +43,8 @@ class TestRemoteProgress(RemoteProgress):
 
     def __init__(self):
         super(TestRemoteProgress, self).__init__()
-        self._seen_lines = list()
-        self._stages_per_op = dict()
+        self._seen_lines = []
+        self._stages_per_op = {}
         self._num_progress_messages = 0
 
     def _parse_progress_line(self, line):
@@ -56,7 +52,6 @@ class TestRemoteProgress(RemoteProgress):
         # Keep it for debugging
         self._seen_lines.append(line)
         rval = super(TestRemoteProgress, self)._parse_progress_line(line)
-        assert len(line) > 1, "line %r too short" % line
         return rval
 
     def line_dropped(self, line):
@@ -97,7 +92,7 @@ class TestRemoteProgress(RemoteProgress):
         assert self._stages_per_op
 
         # must have seen all stages
-        for op, stages in self._stages_per_op.items():  # @UnusedVariable
+        for _op, stages in self._stages_per_op.items():
             assert stages & self.STAGE_MASK == self.STAGE_MASK
         # END for each op/stage
 
@@ -120,7 +115,7 @@ class TestRemote(TestBase):
         self.assertGreater(len(results), 0)
         self.assertIsInstance(results[0], FetchInfo)
         for info in results:
-            self.assertIsInstance(info.note, string_types)
+            self.assertIsInstance(info.note, str)
             if isinstance(info.ref, Reference):
                 self.assertTrue(info.flags)
             # END reference type flags handling
@@ -137,7 +132,7 @@ class TestRemote(TestBase):
         self.assertIsInstance(results[0], PushInfo)
         for info in results:
             self.assertTrue(info.flags)
-            self.assertIsInstance(info.summary, string_types)
+            self.assertIsInstance(info.summary, str)
             if info.old_commit is not None:
                 self.assertIsInstance(info.old_commit, Commit)
             if info.flags & info.ERROR:
@@ -163,7 +158,7 @@ class TestRemote(TestBase):
 
     def _commit_random_file(self, repo):
         # Create a file with a random name and random data and commit it to  repo.
-        # Return the commited absolute file path
+        # Return the committed absolute file path
         index = repo.index
         new_file = self._make_file(osp.basename(tempfile.mktemp()), str(random.random()), repo)
         index.add([new_file])
@@ -256,9 +251,15 @@ class TestRemote(TestBase):
         self.assertEqual(tinfo.ref.commit, rtag.commit)
         self.assertTrue(tinfo.flags & tinfo.NEW_TAG)
 
-        # adjust tag commit
+        # adjust the local tag commit
         Reference.set_object(rtag, rhead.commit.parents[0].parents[0])
-        res = fetch_and_test(remote, tags=True)
+
+        # as of git 2.20 one cannot clobber local tags that have changed without
+        # specifying --force, and the test assumes you can clobber, so...
+        force = None
+        if rw_repo.git.version_info[:2] >= (2, 20):
+            force = True
+        res = fetch_and_test(remote, tags=True, force=force)
         tinfo = res[str(rtag)]
         self.assertEqual(tinfo.commit, rtag.commit)
         self.assertTrue(tinfo.flags & tinfo.TAG_UPDATE)
@@ -323,7 +324,7 @@ class TestRemote(TestBase):
         self._commit_random_file(rw_repo)
         progress = TestRemoteProgress()
         res = remote.push(lhead.reference, progress)
-        self.assertIsInstance(res, IterableList)
+        self.assertIsInstance(res, list)
         self._do_test_push_result(res, remote)
         progress.make_assertion()
 
@@ -383,6 +384,14 @@ class TestRemote(TestBase):
         progress.make_assertion()
         self._do_test_push_result(res, remote)
 
+        # rejected stale delete
+        force_with_lease = "%s:0000000000000000000000000000000000000000" % new_head.path
+        res = remote.push(":%s" % new_head.path, force_with_lease=force_with_lease)
+        self.assertTrue(res[0].flags & PushInfo.ERROR)
+        self.assertTrue(res[0].flags & PushInfo.REJECTED)
+        self.assertIsNone(res[0].local_ref)
+        self._do_test_push_result(res, remote)
+
         # delete new branch on the remote end and locally
         res = remote.push(":%s" % new_head.path)
         self._do_test_push_result(res, remote)
@@ -613,7 +622,7 @@ class TestRemote(TestBase):
         remote.set_url(test2, delete=True)
         self.assertEqual(list(remote.urls), [test1, test3])
         # Testing changing an URL
-        remote.set_url(test3, test2)
+        remote.set_url(test2, test3)
         self.assertEqual(list(remote.urls), [test1, test2])
 
         # will raise: fatal: --add --delete doesn't make sense
@@ -635,7 +644,7 @@ class TestRemote(TestBase):
 
     def test_fetch_error(self):
         rem = self.rorepo.remote('origin')
-        with self.assertRaisesRegex(GitCommandError, "Couldn't find remote ref __BAD_REF__"):
+        with self.assertRaisesRegex(GitCommandError, "[Cc]ouldn't find remote ref __BAD_REF__"):
             rem.fetch('__BAD_REF__')
 
     @with_rw_repo('0.1.6', bare=False)
