@@ -2,15 +2,13 @@
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
 import os
+import shutil
 import sys
-try:
-    from unittest import skipIf
-except ImportError:
-    from unittest2 import skipIf
+from unittest import skipIf
 
 import git
 from git.cmd import Git
-from git.compat import string_types, is_win
+from git.compat import is_win
 from git.exc import (
     InvalidGitRepositoryError,
     RepositoryDirtyError
@@ -81,7 +79,7 @@ class TestSubmodule(TestBase):
         self.failUnlessRaises(InvalidGitRepositoryError, getattr, sm, 'branch')
 
         # branch_path works, as its just a string
-        assert isinstance(sm.branch_path, string_types)
+        assert isinstance(sm.branch_path, str)
 
         # some commits earlier we still have a submodule, but its at a different commit
         smold = next(Submodule.iter_items(rwrepo, self.k_subm_changed))
@@ -267,7 +265,7 @@ class TestSubmodule(TestBase):
             self.failUnlessRaises(ValueError, csm.remove, module=False, configuration=False)
 
             # module() is supposed to point to gitdb, which has a child-submodule whose URL is still pointing
-            # to github. To save time, we will change it to
+            # to GitHub. To save time, we will change it to
             csm.set_parent_commit(csm.repo.head.commit)
             with csm.config_writer() as cw:
                 cw.set_value('url', self._small_repo_url())
@@ -482,9 +480,8 @@ class TestSubmodule(TestBase):
         with sm.config_writer() as writer:
             writer.set_value('path', fp)    # change path to something with prefix AFTER url change
 
-        # update fails as list_items in such a situations cannot work, as it cannot
-        # find the entry at the changed path
-        self.failUnlessRaises(InvalidGitRepositoryError, rm.update, recursive=False)
+        # update doesn't fail, because list_items ignores the wrong path in such situations.
+        rm.update(recursive=False)
 
         # move it properly - doesn't work as it its path currently points to an indexentry
         # which doesn't exist ( move it to some path, it doesn't matter here )
@@ -663,6 +660,24 @@ class TestSubmodule(TestBase):
                                   url=empty_repo_dir, no_checkout=checkout_mode and True or False)
         # end for each checkout mode
 
+    @with_rw_directory
+    def test_list_only_valid_submodules(self, rwdir):
+        repo_path = osp.join(rwdir, 'parent')
+        repo = git.Repo.init(repo_path)
+        repo.git.submodule('add', self._small_repo_url(), 'module')
+        repo.index.commit("add submodule")
+
+        assert len(repo.submodules) == 1
+
+        # Delete the directory from submodule
+        submodule_path = osp.join(repo_path, 'module')
+        shutil.rmtree(submodule_path)
+        repo.git.add([submodule_path])
+        repo.index.commit("remove submodule")
+
+        repo = git.Repo(repo_path)
+        assert len(repo.submodules) == 0
+
     @skipIf(HIDE_WINDOWS_KNOWN_ERRORS,
             """FIXME on cygwin: File "C:\\projects\\gitpython\\git\\cmd.py", line 671, in execute
                 raise GitCommandError(command, status, stderr_value, stdout_value)
