@@ -13,11 +13,9 @@ import subprocess
 import re
 import shutil
 import stat
+from sys import maxsize
 import time
-try:
-    from unittest import SkipTest
-except ImportError:
-    from unittest2 import SkipTest
+from unittest import SkipTest
 
 from gitdb.util import (# NOQA @IgnorePep8
     make_sha,
@@ -34,11 +32,6 @@ from gitdb.util import (# NOQA @IgnorePep8
 from git.compat import is_win
 import os.path as osp
 
-from .compat import (
-    MAXSIZE,
-    defenc,
-    PY3
-)
 from .exc import InvalidGitRepositoryError
 
 
@@ -101,9 +94,8 @@ def rmtree(path):
             func(path)  # Will scream if still not possible to delete.
         except Exception as ex:
             if HIDE_WINDOWS_KNOWN_ERRORS:
-                raise SkipTest("FIXME: fails with: PermissionError\n  %s", ex)
-            else:
-                raise
+                raise SkipTest("FIXME: fails with: PermissionError\n  {}".format(ex))
+            raise
 
     return shutil.rmtree(path, False, onerror)
 
@@ -137,7 +129,7 @@ def join_path(a, *p):
     '/' instead of possibly '\' on windows."""
     path = a
     for b in p:
-        if len(b) == 0:
+        if not b:
             continue
         if b.startswith('/'):
             path += b[1:]
@@ -188,20 +180,15 @@ def assure_directory_exists(path, is_file=False):
 
 
 def _get_exe_extensions():
-    try:
-        winprog_exts = tuple(p.upper() for p in os.environ['PATHEXT'].split(os.pathsep))
-    except:
-        winprog_exts = ('.BAT', 'COM', '.EXE')
-
-    return winprog_exts
+    PATHEXT = os.environ.get('PATHEXT', None)
+    return tuple(p.upper() for p in PATHEXT.split(os.pathsep)) \
+        if PATHEXT \
+        else (('.BAT', 'COM', '.EXE') if is_win else ())
 
 
 def py_where(program, path=None):
     # From: http://stackoverflow.com/a/377028/548792
-    try:
-        winprog_exts = tuple(p.upper() for p in os.environ['PATHEXT'].split(os.pathsep))
-    except:
-        winprog_exts = is_win and ('.BAT', 'COM', '.EXE') or ()
+    winprog_exts = _get_exe_extensions()
 
     def is_exec(fpath):
         return osp.isfile(fpath) and os.access(fpath, os.X_OK) and (
@@ -340,6 +327,16 @@ def finalize_process(proc, **kwargs):
     ## TODO: No close proc-streams??
     proc.wait(**kwargs)
 
+
+def expand_path(p, expand_vars=True):
+    try:
+        p = osp.expanduser(p)
+        if expand_vars:
+            p = osp.expandvars(p)
+        return osp.normpath(osp.abspath(p))
+    except Exception:
+        return None
+
 #} END utilities
 
 #{ Classes
@@ -367,7 +364,7 @@ class RemoteProgress(object):
     re_op_relative = re.compile(r"(remote: )?([\w\s]+):\s+(\d+)% \((\d+)/(\d+)\)(.*)")
 
     def __init__(self):
-        self._seen_ops = list()
+        self._seen_ops = []
         self._cur_line = None
         self.error_lines = []
         self.other_lines = []
@@ -378,101 +375,94 @@ class RemoteProgress(object):
 
         - Lines that do not contain progress info are stored in :attr:`other_lines`.
         - Lines that seem to contain an error (i.e. start with error: or fatal:) are stored
-        in :attr:`error_lines`.
-
-        :return: list(line, ...) list of lines that could not be processed"""
+        in :attr:`error_lines`."""
         # handle
         # Counting objects: 4, done.
-        # Compressing objects:  50% (1/2)   \rCompressing objects: 100% (2/2)   \rCompressing objects: 100% (2/2), done.
-        self._cur_line = line
-        if len(self.error_lines) > 0 or self._cur_line.startswith(('error:', 'fatal:')):
+        # Compressing objects:  50% (1/2)
+        # Compressing objects: 100% (2/2)
+        # Compressing objects: 100% (2/2), done.
+        self._cur_line = line = line.decode('utf-8') if isinstance(line, bytes) else line
+        if self.error_lines or self._cur_line.startswith(('error:', 'fatal:')):
             self.error_lines.append(self._cur_line)
-            return []
+            return
 
-        sub_lines = line.split('\r')
-        failed_lines = list()
-        for sline in sub_lines:
-            # find escape characters and cut them away - regex will not work with
-            # them as they are non-ascii. As git might expect a tty, it will send them
-            last_valid_index = None
-            for i, c in enumerate(reversed(sline)):
-                if ord(c) < 32:
-                    # its a slice index
-                    last_valid_index = -i - 1
-                # END character was non-ascii
-            # END for each character in sline
-            if last_valid_index is not None:
-                sline = sline[:last_valid_index]
-            # END cut away invalid part
-            sline = sline.rstrip()
+        # find escape characters and cut them away - regex will not work with
+        # them as they are non-ascii. As git might expect a tty, it will send them
+        last_valid_index = None
+        for i, c in enumerate(reversed(line)):
+            if ord(c) < 32:
+                # its a slice index
+                last_valid_index = -i - 1
+            # END character was non-ascii
+        # END for each character in line
+        if last_valid_index is not None:
+            line = line[:last_valid_index]
+        # END cut away invalid part
+        line = line.rstrip()
 
-            cur_count, max_count = None, None
-            match = self.re_op_relative.match(sline)
-            if match is None:
-                match = self.re_op_absolute.match(sline)
+        cur_count, max_count = None, None
+        match = self.re_op_relative.match(line)
+        if match is None:
+            match = self.re_op_absolute.match(line)
 
-            if not match:
-                self.line_dropped(sline)
-                failed_lines.append(sline)
-                continue
-            # END could not get match
+        if not match:
+            self.line_dropped(line)
+            self.other_lines.append(line)
+            return
+        # END could not get match
 
-            op_code = 0
-            remote, op_name, percent, cur_count, max_count, message = match.groups()  # @UnusedVariable
+        op_code = 0
+        _remote, op_name, _percent, cur_count, max_count, message = match.groups()
 
-            # get operation id
-            if op_name == "Counting objects":
-                op_code |= self.COUNTING
-            elif op_name == "Compressing objects":
-                op_code |= self.COMPRESSING
-            elif op_name == "Writing objects":
-                op_code |= self.WRITING
-            elif op_name == 'Receiving objects':
-                op_code |= self.RECEIVING
-            elif op_name == 'Resolving deltas':
-                op_code |= self.RESOLVING
-            elif op_name == 'Finding sources':
-                op_code |= self.FINDING_SOURCES
-            elif op_name == 'Checking out files':
-                op_code |= self.CHECKING_OUT
-            else:
-                # Note: On windows it can happen that partial lines are sent
-                # Hence we get something like "CompreReceiving objects", which is
-                # a blend of "Compressing objects" and "Receiving objects".
-                # This can't really be prevented, so we drop the line verbosely
-                # to make sure we get informed in case the process spits out new
-                # commands at some point.
-                self.line_dropped(sline)
-                # Note: Don't add this line to the failed lines, as we have to silently
-                # drop it
-                self.other_lines.extend(failed_lines)
-                return failed_lines
-            # END handle op code
+        # get operation id
+        if op_name == "Counting objects":
+            op_code |= self.COUNTING
+        elif op_name == "Compressing objects":
+            op_code |= self.COMPRESSING
+        elif op_name == "Writing objects":
+            op_code |= self.WRITING
+        elif op_name == 'Receiving objects':
+            op_code |= self.RECEIVING
+        elif op_name == 'Resolving deltas':
+            op_code |= self.RESOLVING
+        elif op_name == 'Finding sources':
+            op_code |= self.FINDING_SOURCES
+        elif op_name == 'Checking out files':
+            op_code |= self.CHECKING_OUT
+        else:
+            # Note: On windows it can happen that partial lines are sent
+            # Hence we get something like "CompreReceiving objects", which is
+            # a blend of "Compressing objects" and "Receiving objects".
+            # This can't really be prevented, so we drop the line verbosely
+            # to make sure we get informed in case the process spits out new
+            # commands at some point.
+            self.line_dropped(line)
+            # Note: Don't add this line to the other lines, as we have to silently
+            # drop it
+            return
+        # END handle op code
 
-            # figure out stage
-            if op_code not in self._seen_ops:
-                self._seen_ops.append(op_code)
-                op_code |= self.BEGIN
-            # END begin opcode
+        # figure out stage
+        if op_code not in self._seen_ops:
+            self._seen_ops.append(op_code)
+            op_code |= self.BEGIN
+        # END begin opcode
 
-            if message is None:
-                message = ''
-            # END message handling
+        if message is None:
+            message = ''
+        # END message handling
 
-            message = message.strip()
-            if message.endswith(self.DONE_TOKEN):
-                op_code |= self.END
-                message = message[:-len(self.DONE_TOKEN)]
-            # END end message handling
-            message = message.strip(self.TOKEN_SEPARATOR)
+        message = message.strip()
+        if message.endswith(self.DONE_TOKEN):
+            op_code |= self.END
+            message = message[:-len(self.DONE_TOKEN)]
+        # END end message handling
+        message = message.strip(self.TOKEN_SEPARATOR)
 
-            self.update(op_code,
-                        cur_count and float(cur_count),
-                        max_count and float(max_count),
-                        message)
-        # END for each sub line
-        self.other_lines.extend(failed_lines)
-        return failed_lines
+        self.update(op_code,
+                    cur_count and float(cur_count),
+                    max_count and float(max_count),
+                    message)
 
     def new_message_handler(self):
         """
@@ -532,8 +522,8 @@ class Actor(object):
     can be committers and authors or anything with a name and an email as
     mentioned in the git log entries."""
     # PRECOMPILED REGEX
-    name_only_regex = re.compile(r'<(.+)>')
-    name_email_regex = re.compile(r'(.*) <(.+?)>')
+    name_only_regex = re.compile(r'<(.*)>')
+    name_email_regex = re.compile(r'(.*) <(.*?)>')
 
     # ENVIRONMENT VARIABLES
     # read when creating new commits
@@ -583,9 +573,8 @@ class Actor(object):
             m = cls.name_only_regex.search(string)
             if m:
                 return Actor(m.group(1), None)
-            else:
-                # assume best and use the whole string as name
-                return Actor(string, None)
+            # assume best and use the whole string as name
+            return Actor(string, None)
             # END special case name
         # END handle name/email matching
 
@@ -599,9 +588,6 @@ class Actor(object):
                                           ('email', env_email, cls.conf_email, default_email)):
             try:
                 val = os.environ[evar]
-                if not PY3:
-                    val = val.decode(defenc)
-                # end assure we don't get 'invalid strings'
                 setattr(actor, attr, val)
             except KeyError:
                 if config_reader is not None:
@@ -668,7 +654,7 @@ class Stats(object):
         """Create a Stat object from output retrieved by git-diff.
 
         :return: git.Stat"""
-        hsh = {'total': {'insertions': 0, 'deletions': 0, 'lines': 0, 'files': 0}, 'files': dict()}
+        hsh = {'total': {'insertions': 0, 'deletions': 0, 'lines': 0, 'files': 0}, 'files': {}}
         for line in text.splitlines():
             (raw_insertions, raw_deletions, filename) = line.split("\t")
             insertions = raw_insertions != '-' and int(raw_insertions) or 0
@@ -794,14 +780,14 @@ class BlockingLockFile(LockFile):
         can never be obtained."""
     __slots__ = ("_check_interval", "_max_block_time")
 
-    def __init__(self, file_path, check_interval_s=0.3, max_block_time_s=MAXSIZE):
+    def __init__(self, file_path, check_interval_s=0.3, max_block_time_s=maxsize):
         """Configure the instance
 
-        :parm check_interval_s:
+        :param check_interval_s:
             Period of time to sleep until the lock is checked the next time.
             By default, it waits a nearly unlimited time
 
-        :parm max_block_time_s: Maximum amount of seconds we may lock"""
+        :param max_block_time_s: Maximum amount of seconds we may lock"""
         super(BlockingLockFile, self).__init__(file_path)
         self._check_interval = check_interval_s
         self._max_block_time = max_block_time_s
@@ -809,7 +795,7 @@ class BlockingLockFile(LockFile):
     def _obtain_lock(self):
         """This method blocks until it obtained the lock, or raises IOError if
         it ran out of time or if the parent directory was not available anymore.
-        If this method returns, you are guranteed to own the lock"""
+        If this method returns, you are guaranteed to own the lock"""
         starttime = time.time()
         maxtime = starttime + float(self._max_block_time)
         while True:
@@ -862,9 +848,12 @@ class IterableList(list):
 
     def __contains__(self, attr):
         # first try identity match for performance
-        rval = list.__contains__(self, attr)
-        if rval:
-            return rval
+        try:
+            rval = list.__contains__(self, attr)
+            if rval:
+                return rval
+        except (AttributeError, TypeError):
+            pass
         # END handle match
 
         # otherwise make a full name search
@@ -915,7 +904,7 @@ class Iterable(object):
 
     """Defines an interface for iterable items which is to assure a uniform
     way to retrieve and iterate items within the git repository"""
-    __slots__ = tuple()
+    __slots__ = ()
     _id_attribute_ = "attribute that most suitably identifies your instance"
 
     @classmethod
@@ -944,8 +933,3 @@ class Iterable(object):
 class NullHandler(logging.Handler):
     def emit(self, record):
         pass
-
-
-# In Python 2.6, there is no NullHandler yet. Let's monkey-patch it for a workaround.
-if not hasattr(logging, 'NullHandler'):
-    logging.NullHandler = NullHandler
